@@ -583,7 +583,7 @@ void NLilyExport::exportStaffs(QString fname, QList<NStaff> *stafflist, exportFr
 
 
 void NLilyExport::writeVoice(int staff_nr,  int voice_nr, NVoice *voi) {
-	NMusElement *elem, *specialElem;
+	NMusElement *elem, *elem2, *specialElem;
 	char *hiddenRepeat = "\\repeat \"volta\" 2 {";
 	int total = 0, i;
 	bool first;
@@ -631,6 +631,8 @@ void NLilyExport::writeVoice(int staff_nr,  int voice_nr, NVoice *voi) {
 	char *lastDynSym = 0;
 	bool textpropset = false;
 	bool fatTextWritten = false;
+	bool inLongacciaccatura = false;
+	bool inA_tura = false;
 	int edgetext = EDGE_TEXT_NOT_SET;
 	int trillendpos = 0;
 	int trilledNotes = 0;
@@ -806,10 +808,36 @@ void NLilyExport::writeVoice(int staff_nr,  int voice_nr, NVoice *voi) {
 					if  (!inGrace) {
 						inGrace = true;
 						strokenGrace = true;
-						out_ << "\\grace { ";
-						if (!NResource::lilyProperties_.lilyVersion2 && elem->getSubType() != INTERNAL_MARKER_OF_STROKEN_GRACE) {
-							strokenGrace = false;
-							out_ << "[ ";
+						inA_tura = inLongacciaccatura = false;
+						if (/* NResource::lilyProperties_.lilyProperties || */ !(elem->status_ & STAT_SLURED)) {
+							out_ << "\\grace { ";
+							if (!NResource::lilyProperties_.lilyVersion2 && elem->getSubType() != INTERNAL_MARKER_OF_STROKEN_GRACE) {
+								strokenGrace = false;
+								out_ << "[ ";
+							}
+						}
+						else { // LilyPond-2.2.x
+							elem2 = voi->getNextPosition();
+							voi->getPrevPosition();
+							if (elem2->getType() == T_CHORD && !(elem2->status_ & STAT_GRACE)) {
+								if (elem->getSubType() == INTERNAL_MARKER_OF_STROKEN_GRACE) {
+									out_ << "\\acciaccatura ";
+								}
+								else {
+									out_ << "\\appoggiatura ";
+								}
+								inGrace = strokenGrace = false;
+								inA_tura = true;
+							}
+							else if (elem2->getType() == T_CHORD && elem->getSubType() == NOTE16_LENGTH && elem2->getSubType() == NOTE16_LENGTH &&
+								(elem->status_ & STAT_BEAMED) && (elem2->status_ & STAT_BEAMED)) {
+								out_ << "\\acciaccatura { ";
+								strokenGrace = false;
+								inLongacciaccatura = true;
+							}
+							else {
+								out_ << "\\grace { ";
+							}
 						}
 					}
 				     }
@@ -817,7 +845,7 @@ void NLilyExport::writeVoice(int staff_nr,  int voice_nr, NVoice *voi) {
 					if (inGrace) {
 						slur_problem_written = false;
 						inGrace = false;
-						if (!strokenGrace) {
+						if (!strokenGrace && !NResource::lilyProperties_.lilyVersion2) {
 							strokenGrace = false;
 							out_ << "] ";
 						}
@@ -1060,7 +1088,7 @@ void NLilyExport::writeVoice(int staff_nr,  int voice_nr, NVoice *voi) {
 					        }
 				     }
 				     if (NResource::lilyProperties_.lilyVersion2) {
-					if (chord->status_ & STAT_SLURED) {
+					if ((chord->status_ & STAT_SLURED) && !inA_tura && !inLongacciaccatura) {
 					  		out_ << " ( ";
 					}
 					if (pendingVolSig) {
@@ -1114,10 +1142,15 @@ void NLilyExport::writeVoice(int staff_nr,  int voice_nr, NVoice *voi) {
 						out_ << "^\\markup{\\large accel.} ";
 						pending_segnos_rirads_accels &= (~(PENDING_ACCELERANDO));
 					}
-					if (chord->status_ & STAT_PART_OF_SLUR) {
-						out_ << ") ";
+					if ((chord->status_ & STAT_PART_OF_SLUR) && !inLongacciaccatura) {
+						if (inA_tura) {
+							inA_tura = false;
+						}
+						else {
+							out_ << ") ";
+						}
 					}
-					if (exportDialog_->lilyBeam->isChecked()) {
+					if (exportDialog_->lilyBeam->isChecked() || inLongacciaccatura) {
 						if ((elem->status_ & STAT_BEAMED) && !inbeam) {
 							out_ << "[ "; inbeam = true;
 						}
@@ -1144,7 +1177,7 @@ void NLilyExport::writeVoice(int staff_nr,  int voice_nr, NVoice *voi) {
 					octaviation = 0;
 					out_ << "#(set-octavation 0) ";
 				     }
-				     if (exportDialog_->lilyBeam->isChecked()) {
+				     if (exportDialog_->lilyBeam->isChecked() || inLongacciaccatura) {
 				     	if (elem->lastBeamed()) {
 						out_ << "] "; inbeam = false;
 				     	}
@@ -1202,8 +1235,8 @@ void NLilyExport::writeVoice(int staff_nr,  int voice_nr, NVoice *voi) {
 					out_ << "\\fatText ";
 				     }
 				     if (inGrace) {
-					inGrace = false;
-					if (!strokenGrace) {
+					inLongacciaccatura = inGrace = false;
+					if (!strokenGrace && !NResource::lilyProperties_.lilyVersion2) {
 						strokenGrace = false;
 						out_ << "] ";
 					}
@@ -1350,11 +1383,11 @@ void NLilyExport::writeVoice(int staff_nr,  int voice_nr, NVoice *voi) {
 			case T_KEYSIG: key = (NKeySig *) elem;
 				     prevElemIsBar = false;
 				     if (inGrace) {
-					inGrace = false;
-					if (!strokenGrace) {
+					if (!strokenGrace && !NResource::lilyProperties_.lilyVersion2) {
 						strokenGrace = false;
 						out_ << "] ";
 					}
+					inGrace = false;
 					out_ << "} ";
 				     }
 				     if (key->isRegular(&kind, &count)) {
@@ -1371,11 +1404,11 @@ void NLilyExport::writeVoice(int staff_nr,  int voice_nr, NVoice *voi) {
 				     break;
 			case T_TIMESIG: timesig = (NTimeSig *) elem;
 					if (inGrace) {
-						inGrace = false;
-						if (!strokenGrace) {
+						if (!strokenGrace && !NResource::lilyProperties_.lilyVersion2) {
 							strokenGrace = false;
 							out_ << "] ";
 						}
+						inGrace = false;
 						out_ << "} ";
 					}
 					if (prevElemIsBar) {
@@ -1396,11 +1429,11 @@ void NLilyExport::writeVoice(int staff_nr,  int voice_nr, NVoice *voi) {
 					out_ << endl; tabsOut();
 				     break;
 			case T_CLEF: if (inGrace) {
-					inGrace = false;
-					if (!strokenGrace) {
+					if (!strokenGrace && !NResource::lilyProperties_.lilyVersion2) {
 						strokenGrace = false;
 						out_ << "] ";
 					}
+					inGrace = false;
 					out_ << "} ";
 				     }
 				     clef = (NClef *) elem;
@@ -1421,7 +1454,7 @@ void NLilyExport::writeVoice(int staff_nr,  int voice_nr, NVoice *voi) {
 				     break;
 			case T_SIGN: if (inGrace) {
 					inGrace = false;
-					if (!strokenGrace) {
+					if (!strokenGrace && !NResource::lilyProperties_.lilyVersion2) {
 						strokenGrace = false;
 						out_ << "] ";
 					}
